# -*- coding: utf-8 -*-

from .config import ConfigParser, ServiceConfig
from .exec_env import DisableEnv, Shell, Docker, StringBuilder
from .service import PackageManager
from .runner import InvokeRunner


class Container(object):
    def __init__(self, role: str):
        config = ConfigParser.find(role)

        self.services = []

        for service in config.services:
            self.services.extend([PackageManager(self.env(service), service)])

    @staticmethod
    def env(config: ServiceConfig):
        if config.env == 'string_builder':
            return StringBuilder(config.string_builder, config.environment, InvokeRunner)
        elif config.env == 'shell':
            return Shell(config.shell, config.environment, InvokeRunner)
        elif config.env == 'docker':
            return Docker(config.docker, config.environment, InvokeRunner)
        else:
            return DisableEnv()

    @classmethod
    def all_role_names(cls):
        return ConfigParser.all_roles()



# -*- coding: utf-8 -*-

import unittest
from kumaoche.exec_env import Docker
from ..config import StubConfig
from ..runner import MocRunner


class TestShell(unittest.TestCase):
    def setUp(self):
        self.config = StubConfig.find('variable_assign_test_role').services[1]
        self.empty_config = StubConfig.find('empty_role').services[1]
        self.env = Docker(self.config.docker, self.config.environment, MocRunner)
        self.empty_env = Docker(self.empty_config.docker, self.empty_config.environment, MocRunner)
        self.src_text = 'git_host:{git_host},git_org:{git_org},git_repo:{git_repo}'
        self.dst_text = 'git_host:github.com,git_org:kumak1,git_repo:variable_assign_test_role'
        self.dst_empty_text = 'git_host:github.com,git_org:kumak1,git_repo:empty_role'

    def test_run(self):
        self.assertEqual(f"cd docker work_dir {self.dst_text} && docker run test_command", self.env.run("test_command"))
        self.assertEqual(f"cd docker work_dir {self.dst_text} && docker run {self.dst_text}", self.env.run(self.src_text))
        self.assertEqual(f"cd path && docker run container command", self.env.run('{container} command', work_dir='path'))
        self.assertEqual(f"cd path && docker run cont command", self.env.run('{container} command', work_dir='path', container='cont'))

    def test_var_assign(self):
        self.assertEqual(self.dst_text, self.env.var_assign(self.src_text))
        self.assertEqual('', self.env.var_assign(''))
        self.assertEqual('changed', self.env.var_assign('{var_test}', {'var_test': 'changed'}))
        self.assertEqual(self.dst_empty_text, self.empty_env.var_assign(self.src_text))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from abc import ABCMeta, abstractmethod


class Runner(metaclass=ABCMeta):
    @abstractmethod
    def run(self, command: str):
        raise NotImplementedError()

    @abstractmethod
    def path_filter(self, path: str):
        raise NotImplementedError()



# -*- coding: utf-8 -*-

import yaml
import os
import sys

from .config_pack import ConfigPack


class ConfigParser(object):

    @classmethod
    def file_path_list(cls):
        return os.environ.get('KUMAOCHE_CONFIG_PATH', os.getcwd() + '/kumaoche_config.yml').split(':')

    @classmethod
    def all_roles(cls, file_path_list=None):
        loaded_yaml = cls.yaml_load(file_path_list)
        loaded_repository_keys = list(loaded_yaml.get('repositories', {}).keys())

        # 順序を維持した重複削除
        return sorted(set(loaded_repository_keys), key=loaded_repository_keys.index)

    @classmethod
    def find(cls, repo: str, file_path_list=None):
        loaded_yaml = cls.yaml_load(file_path_list)
        parsed_yaml = cls.parse_version(loaded_yaml)
        loaded_repository = cls.find_repository_config(parsed_yaml, repo)

        return ConfigPack(loaded_repository)

    @classmethod
    def yaml_load(cls, file_path_list=None):
        if file_path_list is None:
            file_path_list = cls.file_path_list()

        # ファイル存在確認
        for path in file_path_list:
            if not os.path.exists(path):
                print(f'Target config file "{path}" is not exist.')
                sys.exit()

        loaded_yaml = {
            'presets': {
                "environment": {
                    'git_host': 'github.com',
                    'git_org': '',
                    'git_repo': '',
                },
                "shell": {
                    'working_dir': '`ghq root`/{git_host}/{git_org}/{git_repo}',
                    'run': '{command}',
                    'build': '',
                    'up': '',
                    'down': '',
                },
                "docker": {
                    'container_name': '',
                    'working_dir': '`ghq root`/{git_host}/{git_org}/{git_repo}',
                    'run': 'docker-compose run --rm {container} /bin/bash -c "{command}"',
                    'build': 'docker-compose build',
                    'up': 'docker-compose up -d',
                    'down': 'docker-compose down',
                },
                "git": {
                    'lang': 'git',
                    'env': 'shell',
                    'run': '{command}',
                    'setup': 'ghq get git@{git_host}:{git_org}/{git_repo}.git',
                    'update': 'ghq get git@{git_host}:{git_org}/{git_repo}.git && cd `ghq root`/{git_host}/{git_org}/{git_repo} && git switch master && git pull',
                    'repo_dir': '`ghq root`/{git_host}/{git_org}/{git_repo}',
                    'test': '',
                    "shell": {
                        'working_dir': '',
                        'run': '{command}',
                        'build': '',
                        'up': '',
                        'down': '',
                    },
                },
                "php": {
                    'run': '{command}',
                    'setup': 'php -d detect_unicode=Off composer.phar install',
                    'update': 'php -d detect_unicode=Off composer.phar install',
                    'test': './vendor/bin/phpunit',
                },
                "ruby": {
                    'run': '{command}',
                    'setup': 'bundle install',
                    'update': 'bundle install',
                    'test': 'bundle exec rspec --color',
                },
                "node": {
                    'run': '{command}',
                    'setup': 'npm install',
                    'update': 'npm install',
                    'test': 'npm test',
                },
            }
        }

        for file_path in file_path_list:
            with open(file_path) as file:
                new_yaml = yaml.safe_load(file)
                presets = loaded_yaml.get('presets', {})
                loaded_presets = new_yaml.get('presets', {})
                new_presets = {
                    "environment": {**presets.get("environment", {}), **loaded_presets.get("environment", {})},
                    "shell": {**presets.get("shell", {}), **loaded_presets.get("shell", {})},
                    "docker": {**presets.get("docker", {}), **loaded_presets.get("docker", {})},
                    "git": {**presets.get("git", {}), **loaded_presets.get("git", {})},
                    "php": {**presets.get("php", {}), **loaded_presets.get("php", {})},
                    "ruby": {**presets.get("ruby", {}), **loaded_presets.get("ruby", {})},
                    "node": {**presets.get("node", {}), **loaded_presets.get("node", {})},
                }
                loaded_yaml = {**loaded_yaml, **new_yaml, **{'presets': new_presets}}

        return loaded_yaml

    @classmethod
    def parse_version(cls, loaded_yaml):
        version = int(loaded_yaml.get('version', 1))

        if version == 1:
            return loaded_yaml
        else:
            print(f'Yaml version "{version}" is invalid.')
            sys.exit()

    @classmethod
    def find_repository_config(cls, loaded_yaml, repo):
        # 指定リポジトリが設定ファイルに存在するか確認
        loaded_repositories = loaded_yaml.get('repositories', {})
        if repo not in list(loaded_repositories.keys()):
            print(f'Target repository "{repo}" is not exist.')
            sys.exit()

        loaded_repository = loaded_repositories.get(repo, {})

        presets = loaded_yaml.get('presets', {})
        result = {
            "environment": {**presets.get("environment", {}), **cls.safe_get_hash(loaded_repository, "environment")},
            "shell": {**presets.get("shell", {}), **cls.safe_get_hash(loaded_repository, "shell")},
            "docker": {**presets.get("docker", {}), **cls.safe_get_hash(loaded_repository, "docker")},
            "git": {**presets.get("git", {}), **cls.safe_get_hash(loaded_repository, "git")},
            "php": {**presets.get("php", {}), **cls.safe_get_hash(loaded_repository, "php")},
            "ruby": {**presets.get("ruby", {}), **cls.safe_get_hash(loaded_repository, "ruby")},
            "node": {**presets.get("node", {}), **cls.safe_get_hash(loaded_repository, "node")},
            "services": [],
        }

        # 抽出対象に git リポジトリを上書き
        result['environment']['git_repo'] = repo

        for service in cls.safe_get_array(loaded_repository, "services"):
            if service is None:
                continue

            result['services'].extend([{
                "lang": service.get("lang", ""),
                "env": service.get("env", ""),
                "environment": {**result.get("environment", {}), **cls.safe_get_hash(service, "environment")},
                "shell": {**result.get("shell", {}), **cls.safe_get_hash(service, "shell")},
                "docker": {**result.get("docker", {}), **cls.safe_get_hash(service, "docker")},
                "git": {**result.get("git", {}), **cls.safe_get_hash(service, "git")},
                "php": {**result.get("php", {}), **cls.safe_get_hash(service, "php")},
                "ruby": {**result.get("ruby", {}), **cls.safe_get_hash(service, "ruby")},
                "node": {**result.get("node", {}), **cls.safe_get_hash(service, "node")},
            }])

        return result

    @classmethod
    def safe_get_hash(cls, dictionary: {}, key: str):
        if dictionary is None:
            dictionary = {}

        result = dictionary.get(key, {})
        if result is None:
            result = {}

        return result

    @classmethod
    def safe_get_array(cls, dictionary: {}, key: str):
        if dictionary is None:
            dictionary = {}

        result = dictionary.get(key, [])
        if result is None:
            result = []

        return result



# -*- coding: utf-8 -*-

from invoke import run
from .runner import Runner


class InvokeRunner(Runner):
    @classmethod
    def run(cls, command: str):
        return run(command, echo=True, pty=True)

    @classmethod
    def path_filter(cls, path: str):
        return run(f"echo '{path}'", hide=True).stdout.splitlines()[-1]



# -*- coding: utf-8 -*-

from .exec_env_config import ExecEnvConfig
from .docker_config import DockerConfig
from .shell_config import ShellConfig
from .string_builder_config import StringBuilderConfig



# -*- coding: utf-8 -*-

from .service_config import ServiceConfig


class PackageManagerConfig(ServiceConfig):
    def __init__(self, configs: {}):
        super().__init__(configs)

        lang_config = configs.get(self.lang, {})
        self.test = configs.get('test', lang_config.get('test', ''))



# -*- coding: utf-8 -*-

import unittest
from kumaoche.exec_env import DisableEnv
from ..config import StubConfig


class TestShell(unittest.TestCase):
    def setUp(self):
        self.config = StubConfig.find('variable_assign_test_role')
        self.env = DisableEnv()
        self.src_text = 'git_host:{git_host},git_org:{git_org},git_repo:{git_repo},db_host:{db_host},db_port:{db_port},db_user:{db_user},db_database:{db_database}'
        self.dst_text = 'git_host:github.com,git_org:kumak1,git_repo:kumaoche,db_host:db,db_port:3306,db_user:root,db_database:db'

    def test_run(self):
        self.assertEqual(None, self.env.run(self.src_text))

    def test_var_assign(self):
        self.assertEqual(self.src_text, self.env.var_assign(self.src_text))


if __name__ == '__main__':
    unittest.main()
